import 'package:flutter/material.dart';

class ProgressIndicatorOverlay extends StatefulWidget {
  /// Value for the transparent level of the background
  /// replaced with the given `opacity` (which ranges from 0.0 to 1.0).
  ///
  /// Out of range values will have unexpected effects.
  final double opacity;

  /// Which widget do you want to display when the overlay is displayed
  final Widget overlay;

  /// widget to overlaid
  final Widget child;
  const ProgressIndicatorOverlay({
    Key key,
    this.opacity = .3,
    this.overlay,
    this.child,
  }) : super(key: key);

  @override
  ProgressIndicatorOverlayState createState() => ProgressIndicatorOverlayState();
}

class ProgressIndicatorOverlayState extends State<ProgressIndicatorOverlay> {
  bool _isLoading = false;

  /// Show overlay
  void show() => setState(() => _isLoading = true);

  /// Hide overlay
  void hide() => setState(() => _isLoading = false);

  @override
  Widget build(BuildContext context) {
    return Stack(
      children: <Widget>[
        widget.child,
        AnimatedSwitcher(
          switchInCurve: Curves.fastLinearToSlowEaseIn,
          switchOutCurve: Curves.fastLinearToSlowEaseIn,
          child: _isLoading
              ? Material(
            color: Colors.black.withOpacity(widget.opacity),
            child: Center(
              child: widget.overlay ??
                  CircularProgressIndicator(
                    valueColor: AlwaysStoppedAnimation<Color>(Colors.white),
                  ),
            ),
          )
              : const SizedBox(),
          duration: const Duration(milliseconds: 300),
        )
      ],
    );
  }
}
